-- This file is part of FS22_Through_The_Years.
--
-- FS22_Through_The_Years is free software: you can redistribute it and/or
-- modify it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- FS22_Through_The_Years is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-- FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
-- details.
--
-- You should have received a copy of the GNU General Public License along with
-- FS22_Through_The_Years. If not, see <https://www.gnu.org/licenses/>.

source(g_currentModDirectory .. "events/SettingsChangeEvent.lua")
source(g_currentModDirectory .. "events/LoadSettingsForClientEvent.lua")
source(g_currentModDirectory .. "TypeControlOptions.lua")

ThroughTheYears = {}

local function ttyprint(value)
    if value ~= nil then
        print("ThroughTheYears: " .. value)
    else
        print("ThroughTheYears: nil")
    end
end

-- Copied and modified from https://stackoverflow.com/a/27028488
local function dump(o)
    if type(o) == 'table' then
        local s = '{ '
        for k, v in pairs(o) do
            if type(k) ~= 'table' then
                if type(k) ~= 'number' then
                    k = '"' .. k .. '"'
                end
                s = s .. '[' .. k .. '] = ' .. tostring(v) .. ','
            end
        end
        return s .. '} '
    else
        return tostring(o)
    end
end

local function sendSettingsChangedEvent()
    if ThroughTheYears.debugEnabled then
        ttyprint("sendSettingsChangedEvent")
    end

    g_client:getServerConnection():sendEvent(
        SettingsChangeEvent.new(ThroughTheYears.baseYear,
                                ThroughTheYears.baseYearState,
                                ThroughTheYears.minimumOneAvailable,
                                ThroughTheYears.typeControlOptions))
end


function ThroughTheYears.SettingsChanged(baseYear, baseYearState, minimumOneAvailable,
                                         typeControlOptions)
    if ThroughTheYears.debugEnabled then
        ttyprint("SettingsChanged")
    end

	ThroughTheYears.baseYear = baseYear
	ThroughTheYears.baseYearState = baseYearState
	ThroughTheYears.minimumOneAvailable = minimumOneAvailable
	ThroughTheYears.typeControlOptions = typeControlOptions
end

function ThroughTheYears.loadSettingsForClient()
    if ThroughTheYears.debugEnabled then
        ttyprint("loadSettingsForClient")
    end
    g_client:getServerConnection():sendEvent(
        LoadSettingsForClientEvent.new(ThroughTheYears.baseYear,
                                       ThroughTheYears.baseYearState,
                                       ThroughTheYears.minimumOneAvailable,
                                       ThroughTheYears.typeControlOptions))
end

function ThroughTheYears.SettingsChangedClient(baseYear, baseYearState, minimumOneAvailable,
                                               typeControlOptions)
    if ThroughTheYears.debugEnabled then
        ttyprint("SettingsChangedClient")
    end
	ThroughTheYears.baseYear = baseYear
	ThroughTheYears.baseYearState = baseYearState
	ThroughTheYears.minimumOneAvailable = minimumOneAvailable
	ThroughTheYears.typeControlOptions = typeControlOptions
end

function ThroughTheYears:updateItemAvailability(item, state)
    item.showInStore = state
end

function ThroughTheYears:itemMatchesYear(storeItem)
    if storeItem == nil then
        ttyprint("storeItem is nil")
        return false
    end

    if storeItem.specs == nil then
        -- Trigger the getValueFunc of all items to get the year from the
        -- FS22_Vehicle_Years mod
        local desc = g_storeManager:getSpecTypeByName("year")
        if desc == nil then
            ttyprint("Could not get years - Is FS22_Vehicle_Years missing?")
            return false
        end

        StoreItemUtil.loadSpecsFromXML(storeItem)
        desc.getValueFunc(storeItem, nil, nil, nil, nil, nil)
    end

    -- Only modify showInStore for those vehicles that have the year spec to avoid
    -- modifiying any defaults
    if storeItem.specs ~= nil and storeItem.specs.year ~= nil then
        local year = tonumber(storeItem.specs.year)

        -- If year is not a number, then it has failed to parse the year
        if type(year) == "number" then
            if ThroughTheYears.verboseDebugEnabled then
                ttyprint(string.format("Name: %s, year: %s", storeItem.name, storeItem.specs.year))
            end

            -- Only disable vehicles if the category filtering is enabled
            local categoryFilterIsEnabled = false
            if ThroughTheYears.typeControlOptions[storeItem.categoryName] ~= nil then
                local categoryOption = ThroughTheYears.typeControlOptions[storeItem.categoryName]
                categoryFilterIsEnabled = categoryOption.stateIsEnabled
            end

            if categoryFilterIsEnabled and year > ThroughTheYears.realCurrentYear then
                return false
            else
                return true
            end
        else
            ttyprint(string.format("Invalid year: %s", storeItem.specs.year))
        end
    end

    return true
end

local function addToMinimumItems(item, items)
    -- Ensure that is at least one filtered item in the shop
    -- (unless there is an unfiltered item)
    -- Will choose the oldest item, and if there are multiple items of same year,
    -- choose the cheapest
    -- Do not consider any vehicles from extraContent as that may not be available to everyone
    if ThroughTheYears.minimumOneAvailable and
       item.extraContentId == nil and
       item.showInStoreOrig then

        local categoryName = item.categoryName

        -- We group all tractor categories for this
        if string.find(categoryName, "TRACTOR") then
            categoryName = "TRACTOR"
        end

        if item.specs ~= nil and item.specs.year ~= nil then
            if item.showInStore == false then
                local storedItem = items[categoryName]

                if storedItem == nil then
                    items[categoryName] = item
                elseif storedItem.specs.year == nil then
                    -- no-op, unfiltered item already in shop
                elseif storedItem.specs.year > item.specs.year then
                    items[categoryName] = item
                elseif storedItem.specs.year == item.specs.year and
                    storedItem.price > item.price then
                        items[categoryName] = item
                end
            else
                items[categoryName] = item
            end
        else
            if items[categoryName] == nil then
                items[categoryName] = item
            else
                -- Overwrite the stored item by any item that does not have a year spec
                -- This ensures that we do not unnecessary add filtered items if there are items
                -- in the category that is not filtered by year
                if items[categoryName].showInStore == false and item.showInStore then
                    items[categoryName] = item
                end
            end
        end
    end
end

-- Kepe a copy of the original showInStore value so that we can modify it without adding items that
-- should never have been added
local function addShowInStoreCopy()
    if ThroughTheYears.debugEnabled then
        ttyprint("addShowInStoreCopy")
    end

    local shopItems = g_storeManager:getItems();

    for index, item in pairs(shopItems) do
        if item ~= nil then
            item.showInStoreOrig = item.showInStore
        end
    end
end

function ThroughTheYears:updateShowInStore()
    if ThroughTheYears.debugEnabled then
        ttyprint("updateShowInStore")
    end

    local shopItems = g_storeManager:getItems();
    local minimumOneCategoryItems = {}
    local newly_available = 0

    -- Iterate on all storeItems and update the showInStore based on the current year
    -- and the vehicle year
    for index, item in pairs(shopItems) do
        if item ~= nil and item.showInStoreOrig then
            if ThroughTheYears:itemMatchesYear(item) then
                if item.showInStore == false then
                    ThroughTheYears:updateItemAvailability(item, true)
                    newly_available = newly_available + 1
                end
            else
                ThroughTheYears:updateItemAvailability(item, false)
            end

            if ThroughTheYears.minimumOneAvailable then
                addToMinimumItems(item, minimumOneCategoryItems)
            end
        end
    end

    if ThroughTheYears.minimumOneAvailable then
        -- Update showInStore for each filtered item to ensure at least 1 per category exist
        for key, item in pairs(minimumOneCategoryItems) do
            if item.showInStore == false then
                if ThroughTheYears.debugEnabled then
                    ttyprint(string.format("Added %s as only item in category %s", item.name, key))
                end
                item.showInStore = true
            end
        end
    end

    if ThroughTheYears.debugEnabled and newly_available > 0 then
        ttyprint(string.format("%d new vehicles available", newly_available))
    end

    if newly_available ~= 0 then
        local notify_str

        if newly_available > 1 then
            notify_str = string.format("%d new vehicles available", newly_available)
        else
            notify_str = string.format("%d new vehicle available", newly_available)
        end

        g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_OK, notify_str)
    end
end

function ThroughTheYears:yearChanged()
    -- Years change in March

    -- Years in FS22 start at 1, so add the baseYear and subtract 1
    ThroughTheYears.realCurrentYear = ThroughTheYears.baseYear +
        g_currentMission.environment.currentYear - 1

    if ThroughTheYears.debugEnabled then
        ttyprint(string.format("yearChanged: %d (%d %d)", ThroughTheYears.realCurrentYear,
                               ThroughTheYears.baseYear, g_currentMission.environment.currentYear))
    end

    ThroughTheYears:updateShowInStore()
    ThroughTheYears:updateSales()
end

function ThroughTheYears.saveToXMLFile(missionInfo)
    if ThroughTheYears.debugEnabled then
        ttyprint("saveToXMLFile")
    end

	if missionInfo.isValid then
        local xmlFileName = missionInfo.savegameDirectory .. "/" .. ThroughTheYears.optionSavegameFileName
        local xmlFile = XMLFile.create("ThroughTheYears", xmlFileName, "ThroughTheYears")

        if xmlFile ~= nil then
			xmlFile:setInt("ThroughTheYears.baseYear", ThroughTheYears.baseYear)
			xmlFile:setInt("ThroughTheYears.baseYearState", ThroughTheYears.baseYearState)
			xmlFile:setBool("ThroughTheYears.minimumOneAvailable",
                            ThroughTheYears.minimumOneAvailable)

            for category, value in pairs(ThroughTheYears.typeControlOptions) do
                xmlFile:setBool("ThroughTheYears." .. category .. ".stateIsEnabled",
                                value.stateIsEnabled)
            end

            xmlFile:save()
            xmlFile:delete()
        end
    end
end

function ThroughTheYears.loadedMission(mission, node)
    if ThroughTheYears.debugEnabled then
        ttyprint("loadedMission")
    end

    if mission:getIsServer() then
        local xmlFileName = mission.missionInfo.savegameDirectory .. "/" .. ThroughTheYears.optionSavegameFileName

        if mission.missionInfo.savegameDirectory ~= nil and fileExists(xmlFileName) then
            local xmlFile = XMLFile.load("ThroughTheYears", xmlFileName)

            if xmlFile ~= nil then
				ThroughTheYears.baseYear = xmlFile:getInt("ThroughTheYears.baseYear",
                                                          ThroughTheYears.baseYear)
				ThroughTheYears.baseYearState = xmlFile:getInt("ThroughTheYears.baseYearState",
                                                               ThroughTheYears.baseYearState)
				ThroughTheYears.minimumOneAvailable =
                    xmlFile:getBool("ThroughTheYears.minimumOneAvailable",
                                    ThroughTheYears.minimumOneAvailable)

                for category, value in pairs(ThroughTheYears.typeControlOptions) do
                    ThroughTheYears.typeControlOptions[category].stateIsEnabled =
                        xmlFile:getBool("ThroughTheYears." .. category .. ".stateIsEnabled",
                                        ThroughTheYears.typeControlOptions[category].stateIsEnabled)
                end

                xmlFile:delete()
            end
        end
    end

    if mission.cancelLoading then
        return
    end
end

function ThroughTheYears:onBaseYearChanged(state, info, element)
    ThroughTheYears.baseYearState = state
    ThroughTheYears.baseYear = tonumber(info.textElement.text)

    if ThroughTheYears.debugEnabled then
        ttyprint(string.format("Base Year set to %d", ThroughTheYears.baseYear))
    end

    sendSettingsChangedEvent()
    ThroughTheYears:yearChanged()
end

function ThroughTheYears:installBaseYearOption(parentGui)
    -- Clone one of the game's default MultiTextOptionElement options
    -- ("checkDirt") to make life easier
    ThroughTheYears.baseYearOption = parentGui.checkDirt:clone()

    -- Set the MultiTextOptionElement up
    ThroughTheYears.baseYearOption.target = ThroughTheYears
    ThroughTheYears.baseYearOption.id = "baseYearOption"
    ThroughTheYears.baseYearOption:setCallback("onClickCallback", "onBaseYearChanged")
    ThroughTheYears.baseYearOption.elements[4]:setText("Base Year")
    ThroughTheYears.baseYearOption.elements[6]:setText(
        "The base year of the game. Increment to move time forwards or backwards in time manually")

    -- Initalize array used for the options - Use 1950 to the release year of the game
    ThroughTheYears.baseYearOption.yearStringArray = {}
    local loop_cnt = 1
    for year = ThroughTheYears.earliestBaseYear, ThroughTheYears.latestBaseYear do
        ThroughTheYears.baseYearOption.yearStringArray[loop_cnt] = tostring(year)
        loop_cnt = loop_cnt + 1
    end

    ThroughTheYears.baseYearOption:setTexts(ThroughTheYears.baseYearOption.yearStringArray)
    ThroughTheYears.baseYearOption:setState(ThroughTheYears.baseYearState)

    -- Add the MultiTextOptionElement to the list of options
    parentGui.boxLayout:addElement(ThroughTheYears.baseYearOption)

    if ThroughTheYears.debugEnabled then
        ttyprint("installBaseYearOption completed")
    end
end

function ThroughTheYears:onMinimumOneAvailableChanged(state, info, element)
    if ThroughTheYears.debugEnabled then
        ttyprint("onMinimumOneAvailableChanged")
    end

    ThroughTheYears.minimumOneAvailable = state == CheckedOptionElement.STATE_CHECKED

    sendSettingsChangedEvent()
    ThroughTheYears:updateShowInStore()
end

function ThroughTheYears:installMinimumOneAvailableOptions(parentGui)
    -- Clone one of the game's default CheckedOptionElement options
    -- ("checkFruitDestruction") to make life easier
    ThroughTheYears.minimumOneAvailableOption = parentGui.checkFruitDestruction:clone()

    -- Set the option element up
    ThroughTheYears.minimumOneAvailableOption.target = ThroughTheYears
    ThroughTheYears.minimumOneAvailableOption.id = category
    ThroughTheYears.minimumOneAvailableOption:setCallback("onClickCallback",
                                                          "onMinimumOneAvailableChanged")
    ThroughTheYears.minimumOneAvailableOption.elements[4]:setText("Minimum one item available")
    ThroughTheYears.minimumOneAvailableOption.elements[6]:setText(
        "Whether all categories has a least one item available (whichever is oldest)")

    if ThroughTheYears.minimumOneAvailable then
        ThroughTheYears.minimumOneAvailableOption:setState(CheckedOptionElement.STATE_CHECKED)
    else
        ThroughTheYears.minimumOneAvailableOption:setState(CheckedOptionElement.STATE_UNCHECKED)
    end

    -- Add the option element to the list of options
    parentGui.boxLayout:addElement(ThroughTheYears.minimumOneAvailableOption)

    if ThroughTheYears.debugEnabled then
        ttyprint("installMinimumOneAvailableOptions completed")
    end
end

function ThroughTheYears:onTypeControlChanged(state, info, element)
    if ThroughTheYears.debugEnabled then
        ttyprint(string.format("onTypeControlChanged: %s", info.id))
    end

    ThroughTheYears.typeControlOptions[info.id].stateIsEnabled =
        state == CheckedOptionElement.STATE_CHECKED

    sendSettingsChangedEvent()
    ThroughTheYears:updateShowInStore()
    ThroughTheYears:updateSales()
end

function ThroughTheYears:installTypeControlOptions(parentGui)
    for category, value in pairsByKeys(ThroughTheYears.typeControlOptions) do
        -- Clone one of the game's default CheckedOptionElement options
        -- ("checkFruitDestruction") to make life easier
        ThroughTheYears.typeControlOptions[category].option =
            parentGui.checkFruitDestruction:clone()

        -- Set the option element up
        ThroughTheYears.typeControlOptions[category].option.target = ThroughTheYears
        ThroughTheYears.typeControlOptions[category].option.id = category
        ThroughTheYears.typeControlOptions[category].option:setCallback("onClickCallback",
                                                                        "onTypeControlChanged")
        ThroughTheYears.typeControlOptions[category].option.elements[4]:setText(value.name)
        ThroughTheYears.typeControlOptions[category].option.elements[6]:setText(
            string.format("Whether %s are affected by game year", value.name))


        if ThroughTheYears.typeControlOptions[category].stateIsEnabled then
            ThroughTheYears.typeControlOptions[category].option:setState(
                CheckedOptionElement.STATE_CHECKED)
        else
            ThroughTheYears.typeControlOptions[category].option:setState(
                CheckedOptionElement.STATE_UNCHECKED)
        end

        -- Add the option element to the list of options
        parentGui.boxLayout:addElement(ThroughTheYears.typeControlOptions[category].option)
    end

    if ThroughTheYears.debugEnabled then
        ttyprint("installTypeControlOptions completed")
    end
end

function ThroughTheYears:installOptions()
    if ThroughTheYears.initOptionGuiDone == false then
        -- self is a GuiElement

        local title = TextElement.new()
        title:applyProfile("settingsMenuSubtitle", true)
        title:setText("Through the Years")
        self.boxLayout:addElement(title)

        ThroughTheYears:installBaseYearOption(self)
        ThroughTheYears:installMinimumOneAvailableOptions(self)
        ThroughTheYears:installTypeControlOptions(self)

        if ThroughTheYears.debugEnabled then
            ttyprint("installOptions completed")
        end

        ThroughTheYears.initOptionGuiDone = true
    end
end

function ThroughTheYears:drawYearText()
    setTextBold(false)
    setTextAlignment(RenderText.ALIGN_LEFT)
    setTextColor(unpack(GameInfoDisplay.COLOR.TEXT))

    renderText(self.yearTextPositionX, self.yearTextPositionY, self.yearTextSize,
               tostring(ThroughTheYears.realCurrentYear))
end

function ThroughTheYears:storeScaledValues()
    if ThroughTheYears.debugEnabled then
        ttyprint("storeScaledValues")
    end

    self.yearTextSize = self:scalePixelToScreenHeight(GameInfoDisplay.TEXT_SIZE.MONTH)
    local textOffX, textOffY = self:scalePixelToScreenVector(GameInfoDisplay.POSITION.MONTH_TEXT)
    local yearBoxX, yearBoxY = self.yearBox:getPosition()
    self.yearTextPositionX = yearBoxX + self.seasonOverlay.width + textOffX
    self.yearTextPositionY = yearBoxY + textOffY + (self.yearBox:getHeight() -
                                                    self.monthTextSize) * 0.5
end

function ThroughTheYears:createYearBox(superFunc, hudAtlasPath, rightX, bottomY)
    if ThroughTheYears.debugEnabled then
        ttyprint("createYearBox")
    end
    local yearBoxWidth, yearBoxHeight = self:scalePixelToScreenVector(GameInfoDisplay.SIZE.DATE_BOX)
    local iconWidth, iconHeight = self:scalePixelToScreenVector(GameInfoDisplay.SIZE.WEATHER_ICON)
    local marginWidth, _ = self:scalePixelToScreenVector(GameInfoDisplay.SIZE.BOX_MARGIN)
    local centerY = bottomY + self:getHeight() * 0.5

    yearBoxWidth = yearBoxWidth * 1.5
    local sepX = rightX + marginWidth
    local posX = rightX - yearBoxWidth
    local posY = bottomY + (yearBoxHeight - iconHeight) * 0.5

    -- Create box
    local boxOverlay = Overlay.new(nil, posX, bottomY, yearBoxWidth, yearBoxHeight)
    local boxElement = HUDElement.new(boxOverlay)
    self.yearBox = boxElement
    self:addChild(boxElement)
    table.insert(self.infoBoxes, self.yearBox)

    -- Add icon
    local iconOverlay = Overlay.new(g_iconsUIFilename, posX, posY, iconWidth, iconHeight)
    iconOverlay:setUVs(GuiUtils.getUVs({65, 0, 65, 65}))
    iconOverlay:setColor(unpack(GameInfoDisplay.COLOR.ICON))
    self.yearBox:addChild(HUDElement.new(iconOverlay))

    -- Add separator
    local separator = self:createVerticalSeparator(hudAtlasPath, sepX, centerY)
    self.yearBox:addChild(separator)
    self.yearBox.separator = separator

    return superFunc(self, hudAtlasPath, rightX, bottomY)
end

function ThroughTheYears:addSale(itemAdded)
    if ThroughTheYears.debugEnabled then
        ttyprint("addSale")
    end

    local shopItems = g_storeManager:getItems();
    local storeItem = nil

    -- Find storeItem based on the XML file (easiest way to do it) to then compare years
    for _, item in pairs(shopItems) do
        if itemAdded.xmlFilename == item.xmlFilename then
            storeItem = item
            break
        end
    end

    if storeItem == nil then
        return
    end

    -- At this point, since we append the function, the item has already been added, and we need
    -- to remove it again. This is to avoid overwriting the addSale function completely.
    if not ThroughTheYears:itemMatchesYear(item) then
        table.remove(g_currentMission.vehicleSaleSystem.items)
    end
end

function ThroughTheYears:updateSales()
    ttyprint(string.format("updateSales for year %d", ThroughTheYears.realCurrentYear))

    local shopItems = g_storeManager:getItems();
    local indexes_to_remove = {}

    -- Gather indexes for removal - Removing items from the table while iterating it can cause
    -- issues with nil access
    if g_currentMission.vehicleSaleSystem.items ~= nil then
        for _, storeItem in pairs(shopItems) do
            for index, salesItem in pairs(g_currentMission.vehicleSaleSystem.items) do
                if storeItem.xmlFilename == salesItem.xmlFilename and
                   not ThroughTheYears:itemMatchesYear(storeItem) then
                    table.insert(indexes_to_remove, index)
                end
            end
        end
    end

    for _, index in pairs(indexes_to_remove) do
        table.remove(g_currentMission.vehicleSaleSystem.items, index)
    end
end

function ThroughTheYears:init()
    if ThroughTheYears.modActivated == nil then
        ThroughTheYears.debugEnabled = false         -- Set to true to enable debug printing
        ThroughTheYears.verboseDebugEnabled = false  -- Set to true to enable verbose debug printing

        if ThroughTheYears.debugEnabled then
            ttyprint("Initializing")
        end
        g_messageCenter:subscribe(MessageType.YEAR_CHANGED, ThroughTheYears.yearChanged, self)

        ThroughTheYears.modActivated = true
        ThroughTheYears.initOptionGuiDone = false

        ThroughTheYears.earliestBaseYear = 1950
        ThroughTheYears.latestBaseYear = 2022 -- Release year of the game
        ThroughTheYears.baseYear = ThroughTheYears.earliestBaseYear
        ThroughTheYears.baseYearState = 1
        ThroughTheYears.minimumOneAvailable = true
        ThroughTheYears.typeControlOptions = getTypeControlOptions()

        -- Install our options when the settings are opened
        InGameMenuGameSettingsFrame.onFrameOpen = Utils.appendedFunction(
            InGameMenuGameSettingsFrame.onFrameOpen, ThroughTheYears.installOptions)

        ThroughTheYears.optionSavegameFileName = "ThroughTheYears.xml"

        VehicleSaleSystem.addSale = Utils.appendedFunction(
            VehicleSaleSystem.addSale, ThroughTheYears.addSale)

        FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(
            FSCareerMissionInfo.saveToXMLFile, ThroughTheYears.saveToXMLFile)

        if ThroughTheYears.debugEnabled then
            ttyprint("Initialized")
        end
    end
end

function ThroughTheYears:onFinishedLoading()
    if ThroughTheYears.debugEnabled then
        ttyprint("onFinishedLoading")
    end

    addShowInStoreCopy()
    -- Call the yearChanged to set the current year when map is loaded
    -- We call it here during the last stages of loading so that when the player presses "Start"
    -- most of the work has already been done.
    ThroughTheYears.yearChanged()
end

function ThroughTheYears:onStartMission()
    if ThroughTheYears.debugEnabled then
        ttyprint("onStartMission")
    end

    -- Call the yearChanged to set the current year when mission is started, since this is
    -- the first time where the g_currentMission.environment.currentYear is properly set
    ThroughTheYears.yearChanged()
end

-- We have to wait until the map has finished loading to avoid modifying the store before
-- the StoreManager has finished loading
FSBaseMission.onStartMission = Utils.appendedFunction(
    FSBaseMission.onStartMission, ThroughTheYears.onStartMission)

FSBaseMission.onFinishedLoading = Utils.appendedFunction(
    FSBaseMission.onFinishedLoading, ThroughTheYears.onFinishedLoading)

Mission00.loadMission00Finished = Utils.appendedFunction(
    Mission00.loadMission00Finished, ThroughTheYears.loadedMission)

FSBaseMission.onConnectionFinishedLoading = Utils.appendedFunction(
    FSBaseMission.onConnectionFinishedLoading, ThroughTheYears.loadSettingsForClient)

-- Overwrite the createWeatherBox function to inject the yearbox before the weather box
-- This is due to the weather box having some weird behavior if not put last
GameInfoDisplay.createWeatherBox = Utils.overwrittenFunction(GameInfoDisplay.createWeatherBox,
                                                             ThroughTheYears.createYearBox)
GameInfoDisplay.draw = Utils.appendedFunction(GameInfoDisplay.draw, ThroughTheYears.drawYearText)
GameInfoDisplay.storeScaledValues = Utils.appendedFunction(GameInfoDisplay.storeScaledValues,
                                                           ThroughTheYears.storeScaledValues)

ThroughTheYears:init()
